<?php

namespace SM\Service;

use SM\Factory\BillingComparisonFactory;
use SM\Service\FindService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class BillingComparisonService
{
    protected $BillingComparisonFactory;
    protected $findService;
    protected $Environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, BillingComparisonFactory $BillingComparisonFactory, FindService $findService, Environment $Environment, NewCode $newCode)
    {
        $this->connection               = $entityManager->getConnection();
        $this->BillingComparisonFactory = $BillingComparisonFactory;
        $this->FindService              = $findService;
        $this->Environment              = $Environment;
        $this->newCode                  = $newCode;
    }

    public function getBillingComparison($parametros)
    {
        if (isset($parametros["QTPARCELAS"])) {
            // formatando os parametros do contrato para _019_1092_1982
            $contratos = null;
            if (isset($parametros["NRSEQCONTRATO"])) {
                $contratos = $this->FindService->formatarArrayUndescore($parametros["NRSEQCONTRATO"]);
            }

            $parceiro = null;
            if (isset($parametros["NRPARCNEGOCIO"])) {
                $parceiro = $this->FindService->formatarArrayUndescore($parametros["NRPARCNEGOCIO"]);
            }

            $tipoorcamento = null;
            if (isset($parametros["NRTIPOORCAMENTO"])) {
                $tipoorcamento = $this->FindService->formatarArrayUndescore($parametros["NRTIPOORCAMENTO"]);
            }

            $idtpfatura = null;
            if (isset($parametros["IDTPFATURA"])) {
                $idtpfatura = $this->FindService->formatarArrayUndescore($parametros["IDTPFATURA"]);
            }
        
            $validaPeriodoAtual = '';
            $validaPeriodoComp = '';
    
            if (isset($parametros['VALIDAPERIODOATUAL'])) {
                $validaPeriodoAtual = ($parametros['VALIDAPERIODOATUAL'] == '1') ? '' : (($parametros['VALIDAPERIODOATUAL'] == '2') ? ' AND PERIODO IS NULL' : ' AND PERIODO IS NOT NULL');
            }

            if (isset($parametros['VALIDAPERIODOCOMP'])) {
                $validaPeriodoComp = ($parametros['VALIDAPERIODOCOMP'] == '1') ? '' : (($parametros['VALIDAPERIODOCOMP'] == '2') ? ' AND PERIODO_COMP IS NULL' : ' AND PERIODO IS NOT NULL');
            }
        
            $registrosVerificados = '';
            if (isset($parametros['VERIFICARREGISTROS'])) {
                $registrosVerificados = ($parametros['VERIFICARREGISTROS'] == 'T') ? '' : (($parametros['VERIFICARREGISTROS'] == 'S') ?  " AND ((IDREGVERIFICADO = 'S') OR (IDREGVERIFICADO_COMP = 'S'))" : " AND ((IDREGVERIFICADO = 'N') AND (IDREGVERIFICADO_COMP = 'N'))");
            }
        
            return $this->BillingComparisonFactory->getBillingComparison(array(
                "MESCOMPET"           => $parametros["MESCOMPET"],
                "QTPARCELAS"          => $parametros["QTPARCELAS"],
                "NRSEQCONTRATO"       => $contratos,
                "NRPARCNEGOCIO"       => $parceiro,
                "NRTIPOORCAMENTO"     => $tipoorcamento,
                "IDTPFATURA"          => $idtpfatura,
                "VRPERCDIFERENCA_DE"  => isset($parametros["VRPERCDIFERENCA_DE"]) ? $parametros["VRPERCDIFERENCA_DE"] : null,
                "VRPERCDIFERENCA_ATE" => isset($parametros["VRPERCDIFERENCA_ATE"]) ? $parametros["VRPERCDIFERENCA_ATE"] : null,
                "VRDIFERENCA_DE"      => isset($parametros["VRDIFERENCA_DE"]) ? $parametros["VRDIFERENCA_DE"] : null,
                "VRDIFERENCA_ATE"     => isset($parametros["VRDIFERENCA_ATE"]) ? $parametros["VRDIFERENCA_ATE"] : null,
                "IDTIPO"              => $parametros["IDTIPO"]
            ), $validaPeriodoAtual, $validaPeriodoComp, $registrosVerificados);
        }
    }

    public function verificarCompFaturamento($parametros)
    {
        $this->connection->beginTransaction();
       
        try {
            $cdOperador = $this->Environment->getCdOperador();
            $nrOrg = $this->Environment->getNrOrg();
        
            $param = array(
                "CDOPERULTATU"      => $cdOperador,
                "NRORGULTATU"       => $nrOrg,
                "IDREGVERIFICADO"   => isset($parametros['IDREGVERIFICADO']) ? $parametros['IDREGVERIFICADO'] : null,
            );

            $arrayFatura = array();
            $arrayMedicao = array();

            if (isset($parametros['PARAMS'])) {
                foreach ($parametros['PARAMS'] as $linhaGrid) {
                    if ($linhaGrid['IDTIPO'] == 'F') {
                        array_push($arrayFatura, $linhaGrid['NRFATURACONT']);
                        $arrayFormatado = $this->FindService->formatarArray($arrayFatura);
                        $this->BillingComparisonFactory->verificarRegParcela($param, $arrayFormatado);
                    } elseif ($linhaGrid['IDTIPO'] === 'M') {
                        array_push($arrayMedicao, $linhaGrid['NRMEDICAOFATURA']);
                        $arrayFormatado = $this->FindService->formatarArray($arrayMedicao);
                        $this->BillingComparisonFactory->verificarRegMedicao($param, $arrayFormatado);
                    }
                }
            }

            // FAZER VERIFICAÇÃO OU REMOÇÃO DE VERIFICAÇÃO INDIVIDUAL
            if (isset($parametros['IDTIPO'])) {
                if ($parametros['IDTIPO'] === 'F') {
                    $retorno = $this->BillingComparisonFactory->getVerRegParcela(array(
                        "NRFATURACONT" => $parametros['CODIGO']
                    ));
                    $param['IDREGVERIFICADO'] = ($retorno[0]['IDREGVERIFICADO'] === 'N') ? 'S' : 'N';
                    $this->BillingComparisonFactory->verificarRegParcela($param, $parametros['CODIGO']);
                } elseif ($parametros['IDTIPO'] === 'M') {
                    $retorno = $this->BillingComparisonFactory->getVerRegMedicao(array(
                        "NRMEDICAOFATURA" => $parametros['CODIGO']
                    ));
                    $param['IDREGVERIFICADO'] = ($retorno[0]['IDREGVERIFICADO'] === 'N') ? 'S' : 'N';
                    $this->BillingComparisonFactory->verificarRegMedicao($param, $parametros['CODIGO']);
                }
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getContractsBillingComparison()
    {
        return $this->BillingComparisonFactory->getContractsBillingComparison();
    }

    public function getBusinessPartnersBilling()
    {
        return $this->BillingComparisonFactory->getBusinessPartnersBilling();
    }

    public function getBillingScopes()
    {
        return $this->BillingComparisonFactory->getBillingScopes();
    }

    public function billingComparisonOfTheParcelTab($parametros)
    {
        $param = [
            "NRFATURACONT" => $parametros["NR"],
        ];

        return $this->BillingComparisonFactory->billingComparisonOfTheParcelTab(
            $param
        );
    }

    public function billingComparisonOfMeasurementItems($parametros)
    {
        $param = [
            "NRMEDICAOFATURA" => $parametros["NR"],
        ];

        return $this->BillingComparisonFactory->billingComparisonOfMeasurementItems($param);
    }

    public function billingComparisonOfVariableMeasures($parametros)
    {
        $param = [
            "NRMEDICAOFATURA" => $parametros["NR"],
        ];

        return $this->BillingComparisonFactory->billingComparisonOfVariableMeasures(
            $param
        );
    }
}
